/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import java.io.IOException;

import javax.xml.namespace.QName;
import javax.xml.parsers.ParserConfigurationException;

import jakarta.xml.bind.JAXBElement;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ws.server.endpoint.annotation.Endpoint;
import org.springframework.ws.server.endpoint.annotation.PayloadRoot;
import org.springframework.ws.server.endpoint.annotation.RequestPayload;
import org.springframework.ws.server.endpoint.annotation.ResponsePayload;

import de.ozgcloud.eingang.semantik.SemantikAdapter;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Endpoint
@Log4j2
public class IntelliFormEndpoint {

	private static final String NAMESPACE_URI = "http://xmlns.cit.de/intelliform/2009/webservices/backend";

	private static final ObjectFactory objectFactory = new ObjectFactory();

	private static final QName _DepositResponse_QNAME = new QName(NAMESPACE_URI, "depositResponse");
	private static final QName _PrefillResponse_QNAME = new QName(NAMESPACE_URI, "prefillResponse");


	@Autowired
	private IntelliFormMapper intelliFormMapper;
	@Autowired
	private SemantikAdapter semantikAdapter;

	@PayloadRoot(namespace = NAMESPACE_URI, localPart = "deposit")
	@ResponsePayload
	public JAXBElement<DepositResponse> inputFormData(@RequestPayload Deposit deposit)
			throws IOException, ParserConfigurationException {
		try {
			var formData = intelliFormMapper.mapToFormData(deposit.getData());
			semantikAdapter.processFormData(formData);
		} catch (Exception e) {
			LOG.error("Error processing form data", e);
			throw e;
		}

		return new JAXBElement<>(_DepositResponse_QNAME, DepositResponse.class, objectFactory.createDepositResponse());
	}

	@PayloadRoot(namespace = NAMESPACE_URI, localPart = "prefill")
	@ResponsePayload
	public JAXBElement<PrefillResponse> prefillForm(@RequestPayload Prefill prefillRequest) {
		PrefillResponse rsp = objectFactory.createPrefillResponse();
		rsp.setReturn(objectFactory.createPrefillResult());
		rsp.getReturn().setPrimaryDataAttachmentId("HelgeSchneiderId");

		return new JAXBElement<>(_PrefillResponse_QNAME, PrefillResponse.class, rsp);
	}
}
