/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataUtils;
import de.ozgcloud.eingang.semantik.enginebased.EngineBasedSemantikAdapter;
import de.ozgcloud.eingang.semantik.enginebased.afm.intelliform.IntelliFormRepresentationAdapter;
import lombok.RequiredArgsConstructor;

@Component
public class AfmEngineBasedAdapter implements EngineBasedSemantikAdapter {

	@Autowired
	private IntelliFormRepresentationAdapter intelliFormRepresentationAdapter;

	@Autowired
	private List<AfmEngineBasedMapper> mappers;

	@Override
	public FormData parseFormData(FormData formData) {
		var vorgangNummer = formData.getHeader().getVorgangNummer();
		var processedFormData = intelliFormRepresentationAdapter.adaptByRepresentations(formData);

		for (var mapper : mappers) {
			processedFormData = mapper.parseFormData(processedFormData);
		}

		return removeProcessedData(addVorgangNummer(processedFormData, vorgangNummer));
	}

	private FormData addVorgangNummer(FormData formData, String vorgangNummer) {
		return formData.toBuilder()
				.header(formData.getHeader().toBuilder()
						.vorgangNummer(vorgangNummer)
						.build()
				)
				.build();
	}

	private FormData removeProcessedData(FormData formData) {
		return FormDataUtils.from(formData)
				.remove(AfmAntragstellerMapper.POSTFACH_ID)
				.build();
	}

	@Override
	public boolean isResponsible(FormData formData) {
		return intelliFormRepresentationAdapter.isResponsible(formData);
	}
}
