package de.ozgcloud.eingang.xta;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.ws.client.WebServiceClientException;
import org.springframework.ws.client.support.interceptor.ClientInterceptor;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.soap.SoapMessage;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import eu.osci.ws._2014._10.transport.OriginatorsType;
import eu.osci.ws._2014._10.transport.PartyIdentifierType;
import eu.osci.ws._2014._10.transport.PartyType;
import jakarta.validation.Valid;
import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.JAXBElement;
import jakarta.xml.bind.JAXBException;

@Component
class WsHeaderAddingInterceptor implements ClientInterceptor {

	@Autowired
	@Valid
	private XtaProperties properties;

	@Override
	public boolean handleRequest(MessageContext messageContext) throws WebServiceClientException {
		var soapMessage = (SoapMessage) messageContext.getRequest();
		var header = soapMessage.getSoapHeader();

		try {
			JAXBContext context = JAXBContext.newInstance(PartyType.class);
			var marshaller = context.createMarshaller();
			marshaller.marshal(createAuthor(), header.getResult());
		} catch (JAXBException e) {
			throw new TechnicalException("Error on handling Request for adding Header.", e);
		}

		return true;
	}

	JAXBElement<PartyType> createAuthor() {
		eu.osci.ws._2014._10.transport.ObjectFactory objectFactory = new eu.osci.ws._2014._10.transport.ObjectFactory();

		PartyType partyType = new PartyType();
		PartyIdentifierType identifier = new PartyIdentifierType();
		identifier.setValue(properties.getIdentifier());
		partyType.setIdentifier(identifier);

		var origin = new OriginatorsType();
		origin.setAuthor(partyType);

		return objectFactory.createAuthor(partyType);
	}

	@Override
	public boolean handleResponse(MessageContext messageContext) throws WebServiceClientException {
		return true;
	}

	@Override
	public boolean handleFault(MessageContext messageContext) throws WebServiceClientException {
		return true;
	}

	@Override
	public void afterCompletion(MessageContext messageContext, Exception ex) throws WebServiceClientException {
		// nothing to do here
	}

}
