package de.ozgcloud.eingang.xta;

import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;

import de.ozgcloud.eingang.Application;
import lombok.SneakyThrows;

@ActiveProfiles({ "itcase", "local" })
@SpringBootTest(classes = { Application.class, XtaMockServerConfiguration.class })
class XtaServiceITCase {

	@Autowired
	private XtaMockServerConfiguration.XtaMocker xtaMocker;

	@Autowired
	private XtaService xtaService;

	@DisplayName("get messages")
	@Nested
	class TestGetMessages {

		@BeforeEach
		void setup() {
			xtaMocker.setupServer(List.of(
					"mantelantrag_without_anlage.zip",
					"dfoerdermittel_without_anlage.zip",
					"versammlungsanzeige.xml",
					"waffenschein.zip"
			), false);
		}

		@AfterEach
		@SneakyThrows
		void teardown() {
			xtaMocker.teardownServer();
		}

		@DisplayName("should return data for waffenschein, mantelantrag and dfoerdermittel")
		@Test
		void shouldReturnDataForMantelantragAndDfoerdermittel() {
			var formDataItems = xtaService.getMessages().toList();

			assertThat(formDataItems).hasSize(3);
		}

		@DisplayName("should unzip correct number of representations")
		@Test
		void shouldUnzipCorrectNumberOfRepresentations() {
			var firstFormData = xtaService.getMessages().toList().getFirst();

			assertThat(firstFormData.getRepresentations()).hasSize(3);
			assertThat(firstFormData.getNumberOfRepresentations()).isEqualTo(3);
		}

		@DisplayName("should unzip correct number of attachments")
		@Test
		void shouldUnzipCorrectNumberOfAttachments() {
			var firstFormData = xtaService.getMessages().toList().getFirst();

			assertThat(firstFormData.getAttachments()).isEmpty();
		}

		@DisplayName("should have vorgang nummer at correct length")
		@Test
		void shouldHaveVorgangNummerAtCorrectLength() {
			var vorgangNumbers = xtaService.getMessages().map(formData -> formData.getHeader().getVorgangNummer()).toList();

			assertThat(vorgangNumbers).isNotEmpty().allMatch(number -> number.length() == 9);
		}
	}

}
